from synsetslumerger import SynsetsLUMerger
from synsetslumerger2 import SynsetsLUMerger2
from synsetssumomerger import SynsetsSUMOMerger
from synsetssumomerger2 import SynsetsSUMOMerger2
from synsetswndmerger import SynsetsWNDMerger
from synsetswndmerger2 import SynsetsWNDMerger2
from synsetsmsrmerger import SynsetsMSRMerger
from synsetscclmerger import SynsetsCCLMerger
